      PROGRAM SW
c Resolution de l'equation de Saint-Venant 
c avec les schemas numeriques
c 1 Godunov
c 2 Cinetique (avec l'indicatrice) 
c 3 Lax-Friedrich
c 4 Rusanov
      IMPLICIT NONE
      INCLUDE 'common.inc' 
C Declaration
      DOUBLE PRECISION Tf,t
      DOUBLE PRECISION a,b
      DOUBLE PRECISION x(dimmax),dt1,dt2,dt3,dt4
      DOUBLE PRECISION h1(dimmax),h1old(dimmax)
      DOUBLE PRECISION h2(dimmax),h2old(dimmax)
      DOUBLE PRECISION h3(dimmax),h3old(dimmax)
      DOUBLE PRECISION h4(dimmax),h4old(dimmax)

      DOUBLE PRECISION q1(dimmax),q1old(dimmax)
      DOUBLE PRECISION q2(dimmax),q2old(dimmax)
      DOUBLE PRECISION q3(dimmax),q3old(dimmax)
      DOUBLE PRECISION q4(dimmax),q4old(dimmax)

      INTEGER I,CPT,freq


C     Initialisation
      open(1,file='h.plt')
      open(2,file='q.plt')
      eps = 1.0d-12 ! Seuil d'erreur


      n = 100      ! nombre de mailles
      t = 0.0d0    ! temps initial
      Tf= 2.0d0    ! Temps finale
 
      a =  0.0d0   ! Extremite gauche de l'intervalle
      b =  20.0d0  ! Extremite droite de l'intervalle

      cfl = 0.95d0     ! condition de CFL
      dx = (b-a)/(n+1) ! pas d'espace


      I = 0          ! Compteur
      freq = 1       ! sauvegarde toutes les freq iterations
      cpt = 0        ! numero de fichier de sauvergarde        
 
      z0 = 10          ! position de la discontinuité dans la condition de Riemann
      hl =  1          ! état gauche (donnée de Riemann)
      hr =  0.35       ! état droit  (donnée de Riemann)
      ql =  0.0d0      ! état gauche (donnée de Riemann)
      qr =  0.0d0      ! état droit  (donnée de Riemann)

c=================================================================
c Initialisation
      call init(a,x,h1old,q1old) ! initialisation du schema 1
      call init(a,x,h2old,q2old) ! initialisation du schema 2
      call init(a,x,h3old,q3old) ! initialisation du schema 3
      call init(a,x,h4old,q4old) ! initialisation du schema 4
c=================================================================
c=================================================================
      DO WHILE (t .LT. Tf)
c=================================================================
C Sauvegarde
       IF (MOD(I,FREQ).EQ.0) THEN 
         CALL ecrit(x,h1old,q1old,h2old,q2old,h3old,q3old,
     &                h4old,q4old,t,cpt)
         cpt = cpt+1
       ENDIF
c=================================================================
c Calcul du pas de temps
         call calcul_deltat(h1old,q1old,dt1,0) ! Schema GODUNOV
         call calcul_deltat(h2old,q2old,dt2,1) ! Schema CINETIQUE
         call calcul_deltat(h3old,q3old,dt3,0) ! Schema L-F
         call calcul_deltat(h4old,q4old,dt4,0) ! Schema RUSANOV
         dt =min(dt1,dt2,dt3,dt4)
         if (t+dt.ge.Tf) dt = Tf-t
         t =t + dt
      WRITE(*,'(A,F12.6,5X,A,F12.8)'),'TEMPS  = ',T,'Deltat = ',DT
c=================================================================
c Calcul de (h,q)
         CALL calcul_s1(x,h1,q1,h1old,q1old)
         CALL calcul_s2(x,h2,q2,h2old,q2old)
         CALL calcul_s3(x,h3,q3,h3old,q3old)
         CALL calcul_s4(x,h4,q4,h4old,q4old)
c=================================================================
c Maj des conditions aux limites
         CALL cond(h1,q1)
         CALL cond(h2,q2)
         CALL cond(h3,q3)
         CALL cond(h4,q4)
c=================================================================
c Maj de (hold,qold)
         CALL maj(h1,q1,h1old,q1old)
         CALL maj(h2,q2,h2old,q2old)
         CALL maj(h3,q3,h3old,q3old)
         CALL maj(h4,q4,h4old,q4old)
         i = i+1
      ENDDO
c=================================================================
c=================================================================
         CALL ecrit(x,h1old,q1old,h2old,q2old,h3old,q3old,
     &                h4old,q4old,t,cpt)

      close(1)
      close(2)
      stop 'Execution avec succes!!!'     
      end program
